@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # coding: utf-8
 
-# In[3]:
+# In[ ]:
 
 
 try:
@@ -12,7 +12,7 @@ try:
 except:
     pass
 
-version = '0.110'
+version = '0.111'
 
 import re
 import tempfile
@@ -50,7 +50,7 @@ if is_testing():
 
 # # Inspect
 
-# In[4]:
+# In[ ]:
 
 
 import inspect
@@ -62,7 +62,7 @@ def debugFunctionName():
     return inspect.stack()[1][3]
 
 
-# In[5]:
+# In[ ]:
 
 
 debugGetFileName(), debugGetLineNumber(), debugFunctionName()
@@ -70,7 +70,7 @@ debugGetFileName(), debugGetLineNumber(), debugFunctionName()
 
 # # Shell
 
-# In[6]:
+# In[ ]:
 
 
 def shell(cmd):
@@ -83,7 +83,7 @@ def shell(cmd):
     return out
 
 
-# In[7]:
+# In[ ]:
 
 
 shell_template = """/bin/bash << EOF 2>&1
@@ -126,13 +126,13 @@ def bash(cmd, lineList=True, do_print=False, remove_last_empty_line=True):
 
 # # Timing
 
-# In[8]:
+# In[ ]:
 
 
 timeSinceLastCallDict = {}
 
 
-# In[9]:
+# In[ ]:
 
 
 def timeSinceLastCall(name):
@@ -145,14 +145,14 @@ def timeSinceLastCall(name):
     return name + ": " + time_s
 
 
-# In[10]:
+# In[ ]:
 
 
 if is_testing():
     print(timeSinceLastCall('test marker'))
 
 
-# In[11]:
+# In[ ]:
 
 
 import time
@@ -180,7 +180,7 @@ if is_testing():
 
 # # Requiremtns
 
-# In[12]:
+# In[ ]:
 
 
 requirements = []
@@ -212,7 +212,7 @@ import yaml
 
 # # Strings
 
-# In[13]:
+# In[ ]:
 
 
 def sFloat0(f): return "{:0.0f}".format(f)
@@ -242,7 +242,7 @@ if is_testing():
     print(sFloatSci5(1.234567))
 
 
-# In[14]:
+# In[ ]:
 
 
 def strUuid4():
@@ -253,14 +253,14 @@ def strUuid4():
     return uuid
 
 
-# In[15]:
+# In[ ]:
 
 
 if is_testing():
     print(strUuid4())
 
 
-# In[16]:
+# In[ ]:
 
 
 def strRand(n, num=True, upperCase=True, lowerCase=True):
@@ -272,7 +272,7 @@ def strRand(n, num=True, upperCase=True, lowerCase=True):
     return ''.join(random.choice(pool) for _ in range(n))
 
 
-# In[17]:
+# In[ ]:
 
 
 if is_testing():
@@ -281,7 +281,7 @@ if is_testing():
     print(strRand(20, upperCase=False, lowerCase=False))
 
 
-# In[18]:
+# In[ ]:
 
 
 def strMd5Pickle4(obj):
@@ -291,14 +291,14 @@ def strMd5Pickle4(obj):
     return md5.hexdigest()
 
 
-# In[19]:
+# In[ ]:
 
 
 if is_testing():
     print(strMd5Pickle4(['a', 'b']))
 
 
-# In[20]:
+# In[ ]:
 
 
 def strMd5String(s):
@@ -307,7 +307,7 @@ def strMd5String(s):
     return md5.hexdigest()
 
 
-# In[21]:
+# In[ ]:
 
 
 if is_testing():
@@ -316,7 +316,7 @@ if is_testing():
 
 # # Multiprocess / Thread
 
-# In[22]:
+# In[ ]:
 
 
 def mapThread(func, args, n=8):
@@ -341,14 +341,14 @@ if is_testing():
 
 # # Lists
 
-# In[23]:
+# In[ ]:
 
 
 def liChunks(l, chunk_size):
     return [l[i:i + chunk_size] for i in range(0, len(l), chunk_size)]
 
 
-# In[24]:
+# In[ ]:
 
 
 if is_testing():
@@ -357,7 +357,7 @@ if is_testing():
     print(liChunks(['a', 'b', 'c', 'd', 'e', 'f'], 3))
 
 
-# In[25]:
+# In[ ]:
 
 
 def liSplit(a, number_of_splits):
@@ -365,7 +365,7 @@ def liSplit(a, number_of_splits):
     return [a[start:end] for start, end in zip(idxs[:-1], idxs[1:])]
 
 
-# In[26]:
+# In[ ]:
 
 
 if is_testing():
@@ -377,7 +377,7 @@ if is_testing():
     print(liSplit(['a', 'b', 'c', 'd', 'e'], 6))
 
 
-# In[27]:
+# In[ ]:
 
 
 def liJoin(l):
@@ -387,14 +387,14 @@ def liJoin(l):
     return out
 
 
-# In[28]:
+# In[ ]:
 
 
 if is_testing():
     print(liJoin([['a', 'b', 'c'], ['d', 'e', 'f']]))
 
 
-# In[29]:
+# In[ ]:
 
 
 def liFlatten(l):
@@ -411,7 +411,7 @@ def liFlatten(l):
         return l
 
 
-# In[30]:
+# In[ ]:
 
 
 if is_testing():
@@ -421,7 +421,7 @@ if is_testing():
 
 # # List of Strings
 
-# In[31]:
+# In[ ]:
 
 
 regex_cache = {}
@@ -431,35 +431,35 @@ def sListFilter(l, re_include=".*", re_exclude='^\b$'):
     return [e for e in l if regex_cache[re_include].search(e) and not regex_cache[re_exclude].search(e)]
 
 
-# In[32]:
+# In[ ]:
 
 
 if is_testing():
     sListFilter(['good x', 'other good x', 'without the letter before y'])
 
 
-# In[33]:
+# In[ ]:
 
 
 if is_testing():
     print(sListFilter(['good x', 'other good x', 'without the letter before y'], re_include='x'))
 
 
-# In[34]:
+# In[ ]:
 
 
 if is_testing():
     print(sListFilter(['not enough x', 'enough xx', 'enough xx and also z'], re_include='xx+'))
 
 
-# In[35]:
+# In[ ]:
 
 
 if is_testing():
     print(sListFilter(['not enough x', 'enough xx', 'enough xx and also z'], re_include='xx+', re_exclude='z'))
 
 
-# In[36]:
+# In[ ]:
 
 
 def lCsv(l, force_oneline=False):
@@ -475,21 +475,21 @@ def lCsv(l, force_oneline=False):
     return ", ".join([str(e) for e in l])
 
 
-# In[37]:
+# In[ ]:
 
 
 if is_testing():
     print(lCsv(['a', 'b', 1, 3.14]))
 
 
-# In[38]:
+# In[ ]:
 
 
 if is_testing():
     print(lCsv([['a', 'b'], 1, 3.14]))
 
 
-# In[39]:
+# In[ ]:
 
 
 if is_testing():
@@ -498,7 +498,7 @@ if is_testing():
 
 # # Text files
 
-# In[40]:
+# In[ ]:
 
 
 def txtread(path):
@@ -519,7 +519,7 @@ def txtwrite(path, txt):
             return f.write(txt)
 
 
-# In[41]:
+# In[ ]:
 
 
 if is_testing():
@@ -528,21 +528,21 @@ if is_testing():
 
 # # YAML files
 
-# In[42]:
+# In[ ]:
 
 
 def yamlwrite(path, obj):
     txtwrite(path, yaml.dump(obj, sort_keys=False))
 
 
-# In[43]:
+# In[ ]:
 
 
 def yamlread(path):
     return yaml.safe_load(txtread(path=path))
 
 
-# In[44]:
+# In[ ]:
 
 
 if is_testing():
@@ -555,7 +555,7 @@ if is_testing():
 
 # # List of Strings
 
-# In[45]:
+# In[ ]:
 
 
 regex_cache = {}
@@ -567,35 +567,35 @@ def sListFilter(listOfStrings, re_include=".*", re_exclude='^\b$'):
 
 # ### Examples
 
-# In[46]:
+# In[ ]:
 
 
 if is_testing():
     print(sListFilter(['good x', 'other good x', 'without the letter before y']))
 
 
-# In[47]:
+# In[ ]:
 
 
 if is_testing():
     print(sListFilter(['good x', 'other good x', 'without the letter before y'], re_include='x'))
 
 
-# In[48]:
+# In[ ]:
 
 
 if is_testing():
     print(sListFilter(['not enough x', 'enough xx', 'enough xx and also z'], re_include='xx+'))
 
 
-# In[49]:
+# In[ ]:
 
 
 if is_testing():
     print(sListFilter(['not enough x', 'enough xx', 'enough xx and also z'], re_include='xx+', re_exclude='z'))
 
 
-# In[50]:
+# In[ ]:
 
 
 if is_testing():
@@ -604,7 +604,7 @@ if is_testing():
 
 # # GCP Google Cloud Platform
 
-# In[51]:
+# In[ ]:
 
 
 import warnings
@@ -628,7 +628,7 @@ def _gsBlob(path):
     return blob
 
 
-# In[52]:
+# In[ ]:
 
 
 def gsGsToHttp(path):
@@ -638,7 +638,7 @@ def gsHttpToGs(path):
     return path.replace("https://storage.cloud.google.com/", "gs://").split("?")[0] # "?" is for possible "?authuser=0"
 
 
-# In[53]:
+# In[ ]:
 
 
 def gsLs(path):
@@ -648,7 +648,7 @@ def gsLs(path):
     return ['gs://' + bucket + '/' + blob.name for blob in client.list_blobs(prefix=prefix)]
 
 
-# In[54]:
+# In[ ]:
 
 
 def gsExists(path):
@@ -658,7 +658,7 @@ def gsExists(path):
     return client.blob(prefix).exists()
 
 
-# In[55]:
+# In[ ]:
 
 
 def gsUploadFile(gsPath, path):
@@ -666,7 +666,7 @@ def gsUploadFile(gsPath, path):
     blob.upload_from_filename(path)
 
 
-# In[56]:
+# In[ ]:
 
 
 def gsTxtread(path):
@@ -678,7 +678,7 @@ def gsTxtread(path):
     return blob.download_as_string().decode()
 
 
-# In[57]:
+# In[ ]:
 
 
 def gsTxtwrite(path, s):
@@ -686,7 +686,7 @@ def gsTxtwrite(path, s):
     return blob.upload_from_string(s)
 
 
-# In[58]:
+# In[ ]:
 
 
 def gsImwrite(path, img):
@@ -695,7 +695,7 @@ def gsImwrite(path, img):
     blob.upload_from_string(encimg.tobytes())
 
 
-# In[59]:
+# In[ ]:
 
 
 def gsImread(path):
@@ -704,7 +704,7 @@ def gsImread(path):
     return cv2.imdecode(raw, 1)[:, :, [2, 1, 0]]
 
 
-# In[60]:
+# In[ ]:
 
 
 def gsPklWrite(path, obj):
@@ -712,7 +712,7 @@ def gsPklWrite(path, obj):
     return blob.upload_from_string(pickle.dumps(obj, protocol=4))
 
 
-# In[61]:
+# In[ ]:
 
 
 def gsPklRead(path):
@@ -722,7 +722,7 @@ def gsPklRead(path):
 
 # # Pickle
 
-# In[62]:
+# In[ ]:
 
 
 def pklwrite(path, obj):
@@ -735,7 +735,7 @@ def pklwrite(path, obj):
             return pickle.dump(obj, f, protocol=4)
 
 
-# In[63]:
+# In[ ]:
 
 
 def pklread(path):
@@ -746,7 +746,7 @@ def pklread(path):
             return pickle.load(f)
 
 
-# In[64]:
+# In[ ]:
 
 
 if is_testing():
@@ -759,7 +759,7 @@ if is_testing():
 
 # # Images
 
-# In[65]:
+# In[ ]:
 
 
 def _imreadSsh(path):
@@ -778,13 +778,14 @@ def imread(path):
         img = cv2.imdecode(raw, cv2.IMREAD_COLOR)
     elif 'gs://' in path:
         return gsImread(path)
-    elif os.path.isfile(path):
-        img = cv2.imread(path, cv2.IMREAD_COLOR)
+    elif os.path.isfile(os.path.expanduser(path)):
+        img = cv2.imread(os.path.expanduser(path), cv2.IMREAD_COLOR)
     else:
         raise RuntimeError("Image not found: ", path)
     return img[:, :, [2, 1, 0]] # This converts the cv2 colors to RGB
 
 def imsread(paths):
+    paths = [os.path.expanduser(p) for p in paths]
     if isinstance(paths, str) and os.path.isdir(paths):
         return [imread(path) for path in fiFindByWildcard(os.path.join(paths, "*.png"))]
     assert isinstance(paths, list) or isinstance(paths, tuple), type(paths)
@@ -807,6 +808,8 @@ def imwrite(path=None, img=None, jpeg_quality=95, print_assumptions=True):
 
     if "gs://" in path:
         gsImwrite(path, img)
+    
+    path = os.path.expanduser(path)
     os.makedirs(os.path.dirname(path), exist_ok=True)
     
     if path[-4:] == '.jpg' or path[-4:] == '.jpeg':
@@ -840,7 +843,7 @@ def impad(img, top=0, bottom=0, left=0, right=0, color=255, mode='constant'):
         raise RuntimeError(f"Mode '{mode}' not found")
 
 
-# In[66]:
+# In[ ]:
 
 
 def imscaleNN(img, s):
@@ -852,7 +855,7 @@ def imscaleBic(img, s):
 
 
 
-# In[67]:
+# In[ ]:
 
 
 def imshow(array, scale=1, force_nb=False):
@@ -867,14 +870,14 @@ def imshow(array, scale=1, force_nb=False):
         cv2.waitKey()
 
 
-# In[68]:
+# In[ ]:
 
 
 def imNewNoise(height, width, scaleNN=1):
     return imscaleNN((255 * np.random.rand(height, width, 3)).astype(np.uint8), scaleNN)
 
 
-# In[69]:
+# In[ ]:
 
 
 if is_testing():
@@ -882,7 +885,7 @@ if is_testing():
     imshow(impad(img, bottom=10, right=10, mode='reflect'))
 
 
-# In[70]:
+# In[ ]:
 
 
 def imCropUniformBorder(img, color=255):
@@ -921,7 +924,7 @@ if is_testing():
                         raise RuntimeError('Crop did not work for ', i, j, k, l)
 
 
-# In[71]:
+# In[ ]:
 
 
 def imDecodeBytes(imBytes):
@@ -932,7 +935,7 @@ def imDecodeBytes(imBytes):
     return img
 
 
-# In[72]:
+# In[ ]:
 
 
 #if is_testing():
@@ -940,14 +943,14 @@ def imDecodeBytes(imBytes):
 #    imshow(img)
 
 
-# In[73]:
+# In[ ]:
 
 
 #if is_testing():
 #    imshow('http://via.placeholder.com/70.png')
 
 
-# In[74]:
+# In[ ]:
 
 
 if is_testing():
@@ -966,7 +969,7 @@ if is_testing():
         imshow(out_large_interbic)
 
 
-# In[75]:
+# In[ ]:
 
 
 def imRepeat(img, height, width):
@@ -980,7 +983,7 @@ def imRepeat(img, height, width):
     return img
 
 
-# In[76]:
+# In[ ]:
 
 
 if is_testing():
@@ -988,33 +991,33 @@ if is_testing():
     imshow(imRepeat(img_random_color_3x5, 7, 11), scale=10)
 
 
-# In[77]:
+# In[ ]:
 
 
 def imNewWhite(height, width):
     return np.ones((height, width, 3)).astype(np.uint8) * 255
 
 
-# In[78]:
+# In[ ]:
 
 
 if is_testing(): imshow(imNewWhite(10, 500) - 30)
 
 
-# In[79]:
+# In[ ]:
 
 
 def imNewBlack(height, width):
     return np.zeros((height, width, 3)).astype(np.uint8)
 
 
-# In[80]:
+# In[ ]:
 
 
 if is_testing(): imshow(imNewBlack(10, 500))
 
 
-# In[81]:
+# In[ ]:
 
 
 def imAddNoiseGauss(img, std):
@@ -1026,7 +1029,7 @@ def imAddNoiseGauss(img, std):
 if is_testing(): imshow(imAddNoiseGauss(imNewWhite(150, 150), 30))
 
 
-# In[82]:
+# In[ ]:
 
 
 if is_testing():
@@ -1034,7 +1037,7 @@ if is_testing():
     imshow('/tmp/img.jpg')
 
 
-# In[83]:
+# In[ ]:
 
 
 def imJpgDegradation(img, quality):
@@ -1047,7 +1050,7 @@ def imJpgDegradation(img, quality):
 #if is_testing(): imshow(imJpgDegradation(imread('http://via.placeholder.com/150.png'), 15))
 
 
-# In[84]:
+# In[ ]:
 
 
 def imCropCenter(img, size):
@@ -1062,35 +1065,35 @@ def imCropCenter(img, size):
     return img[h_start:h_end, w_start:w_end]
 
 
-# In[85]:
+# In[ ]:
 
 
 #if is_testing(): 
 #    imshow(imCropCenter(imread('http://via.placeholder.com/40.png'), 10))
 
 
-# In[86]:
+# In[ ]:
 
 
 #if is_testing(): 
 #    imshow(imCropCenter(imread('http://via.placeholder.com/40.png'), 49))
 
 
-# In[87]:
+# In[ ]:
 
 
 #if is_testing(): 
 #    imshow(imCropCenter(imread('http://via.placeholder.com/40.png'), 50))
 
 
-# In[88]:
+# In[ ]:
 
 
 #if is_testing(): 
 #    imshow(imCropCenter(imread('http://via.placeholder.com/40.png'), 100))
 
 
-# In[89]:
+# In[ ]:
 
 
 def imCropRandom(img, size, auto_size=True):
@@ -1113,7 +1116,7 @@ def imCropRandom(img, size, auto_size=True):
     return img[h_start:h_end, w_start:w_end]
 
 
-# In[90]:
+# In[ ]:
 
 
 #if is_testing(): 
@@ -1123,7 +1126,7 @@ def imCropRandom(img, size, auto_size=True):
 #    imshow(imCropRandom(imread('http://via.placeholder.com/40.png'), 41))
 
 
-# In[91]:
+# In[ ]:
 
 
 def draw_rect(img, h_beg, h_end, w_beg, w_end, line_width, color=(0, 0, 0)):
@@ -1205,7 +1208,7 @@ def imCropRandomSeed(img, sizes, idx, crop_idx, use_crop=True, use_minimap=False
         return crop
 
 
-# In[92]:
+# In[ ]:
 
 
 if is_testing():
@@ -1216,7 +1219,7 @@ if is_testing():
     imshow(imCropRandomSeed(None, (10, 20), idx=0, crop_idx=0, use_crop=True, use_minimap=False), scale=3)
 
 
-# In[93]:
+# In[ ]:
 
 
 def imGallery(imgs, pad=0, n_cols=None, n_rows=None, ratio=None):
@@ -1269,7 +1272,7 @@ def imGallery(imgs, pad=0, n_cols=None, n_rows=None, ratio=None):
     return out
 
 
-# In[94]:
+# In[ ]:
 
 
 if is_testing():
@@ -1293,7 +1296,7 @@ if is_testing():
     imshow(imGallery(listOfDummyImagesBlackToWhite[:3], pad=1, ratio=0))
 
 
-# In[95]:
+# In[ ]:
 
 
 #if is_testing():
@@ -1301,7 +1304,7 @@ if is_testing():
 #    imshow(imGallery([img40x20, img40x20, img40x20], pad=1))
 
 
-# In[96]:
+# In[ ]:
 
 
 def imAddTextHeader(img, text, h=None, size=None):
@@ -1336,7 +1339,7 @@ if is_testing():
     imshow(imAddTextHeader(imNewBlack(100, 100), 'black image'))
 
 
-# In[97]:
+# In[ ]:
 
 
 if is_testing():
@@ -1346,7 +1349,7 @@ if is_testing():
 
 # # Video
 
-# In[98]:
+# In[ ]:
 
 
 #def imgsToVid(imgs, file_name='out.avi'):
@@ -1366,7 +1369,7 @@ if is_testing():
 #imgsToVid(frames, 'test.avi')
 
 
-# In[99]:
+# In[ ]:
 
 
 #ffmpeg -framerate 1/5 -i img%03d.png -c:v libx264 -r 30 -pix_fmt yuv420p out.mp4
@@ -1375,7 +1378,7 @@ if is_testing():
 # ffmpeg -framerate 1/5 -pattern_type glob -i '*.png' -c:v libx264 -r 30 -pix_fmt yuv420p -vf "crop=trunc(iw/2)*2:trunc(ih/2)*2"  ../out.mp4
 
 
-# In[100]:
+# In[ ]:
 
 
 def imsToMp4(path, imgs, time_per_image, frame_rate_output_video=30, verbose=0):
@@ -1395,7 +1398,7 @@ if is_testing():
 
 # # pytorch
 
-# In[101]:
+# In[ ]:
 
 
 def ptT(img_np_uint8):
@@ -1428,6 +1431,7 @@ if is_testing():
 
 
 def _fiFindByWildcardSsh(wildcard):
+    wildcard = os.path.expanduser(wildcard)
     login, path = wildcard.split(':')
     paths = bash(f"ssh {login} 'ls {path}'")
     return [f'{login}:{p}' for p in natsort.natsorted(paths)]
@@ -1941,7 +1945,10 @@ if is_testing():
     package_files['lx/__init__.py'] = txtread("lx.py")
     
     output = shell("""python3 -c 'import importlib; lx = importlib.import_module(name=".", package="lx"); print(lx.version)'""")
-    raise RuntimeError(output.strip(), 'Version correct: ', output.strip() == version, "(", output.strip(), version, ")")
+    if output.strip() != version:
+        raise RuntimeError(output.strip(), 'Version correct: ', output.strip() == version, "(", output.strip(), version, ")")
+    
+    
 
 
 # In[ ]:
